@@ -23,7 +23,7 @@ from lib389.replica import Replicas
 # We need to be a factor to the backend monitor
 from lib389.monitor import MonitorBackend
 from lib389.index import Index, Indexes, VLVSearches, VLVSearch
-from lib389.tasks import ImportTask, ExportTask, Tasks
+from lib389.tasks import ImportTask, ExportTask, CleanAllRUVTask, Tasks
 from lib389.encrypted_attributes import EncryptedAttr, EncryptedAttrs
 
 
